function gsim=gene_similarity(data,trmsim,param)
% Calculate gene similarity
% data(genes,terms) - annotation matrix
% trmsim(terms,terms) - GO terms similarity matrix
% param - type of the method
% 'm1' - avg_sum -> 1/(m*n) * sum(sim(t1,t2))
% 'm2' - max(sim(t1,t2))
% 'm3' - avg_max -> 1/(m+n) * ( sum_k(max(sim(tk,tp)) + sum_p(max(sim(tk,tp)) )

[n1,n2]=size(data);
total=sum(sum(data));
gsim=ones(n1,n1);

if param == 'm1'                    % avg_sum -> 1/(m*n) * sum(sim(t1,t2))
for i=1:n1-1,
    m=sum(data(i,:));               % num of annotations for a gene i
    ti=find(data(i,:));             % annotated terms
    for j=i+1:n1,
        n=sum(data(j,:));           % num of annotations for a gene j
        tj=find(data(j,:));         % annotated terms
        sk=0;
        sl=0;
        tss=0;
        for k=1:numel(ti),                          % for each term to which i is annotated
            tss=tss+sum(trmsim(ti(k),tj),2);
        end;
        gsim(i,j)=(1/(m*n))*tss;
        gsim(j,i)=gsim(i,j);
    end;
    fprintf('%d ',i);
    if floor(i/100)*100==i,
        fprintf('\n');
    end;
end;
elseif param == 'm2'                    % max(sim(t1,t2))
for i=1:n1-1,
    m=sum(data(i,:));               % num of annotations for a gene i
    ti=logical(data(i,:));             % annotated terms
    for j=i+1:n1,
        n=sum(data(j,:));           % num of annotations for a gene j
        tj=logical(data(j,:));         % annotated terms
        tsmx=0;
        tsnx=0;
        tsnx=max(trmsim(ti,tj));         % find max term similarity with the terms of gene i
        gsim(i,j)=max(tsnx);
        gsim(j,i)=gsim(i,j);
    end;
    fprintf('%d ',i);
    if floor(i/100)*100==i,
        fprintf('\n');
    end;
end;
elseif param == 'm3'                % avg_max -> 1/(m+n) * ( sum_k(max(sim(tk,tp)) + sum_p(max(sim(tk,tp)) )
for i=1:n1-1,
    m=sum(data(i,:));               % num of annotations for a gene i
    ti=logical(data(i,:));             % annotated terms
    for j=i+1:n1,
        n=sum(data(j,:));           % num of annotations for a gene j
        tj=logical(data(j,:));         % annotated terms
        tsmx=0;
        tsnx=0;
        tsnx=max(trmsim(ti,tj));         % find max term similarity with the terms of gene i
        tsmx=max(trmsim(tj,ti));         % find max term similarity with the terms of gene j
        gsim(i,j)=(sum(tsmx)+sum(tsnx))/(m+n);
        gsim(j,i)=gsim(i,j);
    end;
    fprintf('%d ',i);
    if floor(i/100)*100==i,
        fprintf('\n');
    end;
end;
elseif param == 'm4'                % M fuzzy measure (Jaccard similarity/DISTANCE)
    for i=1:n1,
        for j=1:n1,
            a=and(data(i,:),data(j,:));
            b=or(data(i,:),data(j,:));
            gsim(i,j)=termsim(i,a)./(a+b);
        end;
    end;
    
    for i=1:n1,
        gsim(i,i)=0;
    end;

end;
        
            
save('gene_sim.mat','gsim');